// This is free and unencumbered software released into the public
// domain.  For more information, see <http://unlicense.org> or the
// accompanying UNLICENSE file.

package pers

import (
	"fmt"
	"reflect"

	"git.sr.ht/~nelsam/hel/sel"
	"git.sr.ht/~nelsam/hel/vegr/ret"
)

// Return will enqueue a normal, non-panicking return on a mock method.
//
// Return panics if:
//   - the passed in mock value is not a valid mock field (a channel or
//     struct of channels).
//   - the passed in args cannot be returned on the mock field.
//   - the passed in mock value is already full and sending another
//     return value would block.
func Return(mock any, args ...any) {
	panicIdx := ret.PanickerFieldIdx(mock)
	if panicIdx >= 0 {
		if panicIdx > ret.BlockerFieldIdx(mock) {
			panicIdx--
		}
		afterPanic := append([]any(nil), args[panicIdx:]...)
		args = append(args[:panicIdx], nil)
		args = append(args, afterPanic...)
	}
	cases, err := sel.Cases(reflect.SelectSend, mock, args...)
	if err != nil {
		panic(fmt.Errorf("pers: returning %v on mock (%T) is not possible: %w", args, mock, err))
	}
	def := reflect.SelectCase{
		Dir: reflect.SelectDefault,
	}
	for _, c := range cases {
		defChoice := 1
		chosen, _, _ := reflect.Select([]reflect.SelectCase{c, def})
		if chosen == defChoice {
			panic(fmt.Sprintf("pers: returning %v on mock (%T) would block [hint: increase channel buffer size]", args, mock))
		}
	}
}
