package ret

import "reflect"

// Blocker is a type which can be used in hel mocks to block method returns. When
// the return is unblocked, the channel is expected to have a value buffered.
type Blocker chan struct{}

// BlockerFieldIdx returns the index of the Blocker field, if any.
func BlockerFieldIdx(mock any) int {
	v := reflect.ValueOf(mock)
	if v.Kind() == reflect.Ptr {
		v = v.Elem()
	}
	if v.Kind() != reflect.Struct {
		return -1
	}
	for i := 0; i < v.NumField(); i++ {
		if _, ok := v.Field(i).Interface().(Blocker); ok {
			return i
		}
	}
	return -1
}
